/* IBM_PROLOG_BEGIN_TAG                                                   */
/* This is an automatically generated prolog.                             */
/*                                                                        */
/*                                                                        */
/*                                                                        */
/* Licensed Materials - Property of IBM                                   */
/*                                                                        */
/* Restricted Materials of IBM                                            */
/*                                                                        */
/* (C) COPYRIGHT International Business Machines Corp. 2008,2013          */
/* All Rights Reserved                                                    */
/*                                                                        */
/* US Government Users Restricted Rights - Use, duplication or            */
/* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.      */
/*                                                                        */
/* IBM_PROLOG_END_TAG                                                     */

/****************************************************************************
 *
 *   Data structures used to handle HPM counter data
 *
 ****************************************************************************/

#ifndef HPM_DATA_H
#define HPM_DATA_H

#define TRUE_ 1
#define FALSE_ 0

#define MPI_TASK_ID_NOT_AVAILABLE (-1)
#define INSTR_ID_NOT_AVAILABLE    (-1)

/*
 * The struct hpm_event_info records counter information in
 * an architecture independent way.
 *
 * This is used to keep the value and name information of a 
 * single event counter.
 */

typedef struct {
  int event_id;                 /* The value of this id is architecture dependent ! */
  int group_id;                 /* Identifier of the group for this event.          */
  int group_pos;                /* Position of this counter inside the group        */
  char *name;                   /* These are uninitialized pointers                 */
  char *short_name;             /* They will be intialized only when needed         */
  long long value;              /* Accumulated value                                */
  long long start_value;        /* Initial value at start of event counting         */
} hpm_event_info;

/*
 * The struct hpm_data_item gathers num_data event values, along with
 * the description of the values and a timing information that applies
 * to all event values.
 *
 * The timing information is given in seconds and stored 
 * in a vector of doubles with the following meanings.
 *
 * In case of multiplexing of several counter settings, the values
 * are stored in several hpm_data_items.
 *
 * An hpm_data_item can also carry data from a call to getrusage().
 * In this case, only the structure members times[HPM_TIME_USER],
 * times[HPM_TIME_SYSTEM], num_data, data[*].value, data[*].short_name
 * and data[*].name carry meaningful data and the flag is_rusage is set.
 */

/*      NAME                INDEX   DESCRIPTION                             */

#define HPM_NTIM 8
#define HPM_TIME_WALLCLOCK      0       /* Wall clock time                      */
#define HPM_TIME_CYCLE          1       /* User time generated from ticks       */
#define HPM_TIME_USER           2       /* User time as reported by rusage()    */
#define HPM_TIME_SYSTEM         3       /* System time as reported by rusage()  */
#define HPM_TIME_START          4       /* Start time stamp (volatile)          */
#define HPM_TIME_STOP           5       /* Stop time stamp (volatile)           */
#define HPM_TIME_OVERHEAD       6       /* Overhead time for counter start/stop */
#define HPM_TIME_INIT           7       /* Overhead time for initialization     */

/*
 * These defines will be set depending on the group type
 * Each of the group types may have a different number of members
 */

#define HPM_GROUP_TYPE_UNSET   (0)
#define HPM_GROUP_TYPE_CPU     (1)
#define HPM_GROUP_TYPE_HFI     (2)
#define HPM_GROUP_TYPE_CAU     (3)
#define HPM_GROUP_TYPE_RUSAGE  (4)

typedef struct {
  int num_data;                 /* Number event info entries            */
  hpm_event_info *data;         /* Event info list                      */
  double times[HPM_NTIM];       /* Timing information                   */
  int group_type;               /* cpu, hfi, cau, rusage                */
  int mpi_task_id;              /* MPI task id if available             */
  int instr_id;                 /* id of the instrumented code          */
                                /* section if applicable (-1 otherwise) */
  int count;                    /* number of entries/exits to the       */
                                /* instrumented section (if applicable) */
  int is_exclusive;             /* these are excusive values            */
  int xml_element_id;           /* arbitrary integer to identfy         */
                                /* hpm_data_items that translate into   */
                                /* identical XML element shapes         */
  char *description;            /* Description for this data item       */
  char *xml_descr;              /* Descriptive header for XML output    */
} hpm_data_item;

typedef hpm_data_item *hpm_event_vector;

#endif
